@@ -22,7 +22,8 @@
 //! [![docs.rs](https://docs.rs/windmark/badge.svg)](https://docs.rs/windmark)
 //! [![github.com](https://github.com/gemrest/windmark/actions/workflows/check.yaml/badge.svg?branch=main)](https://github.com/gemrest/windmark/actions/workflows/check.yaml)
 //!
-//! Windmark is an elegant and highly performant, async Gemini server framework.
+//! Windmark is an elegant and highly performant, async Gemini server framework
+//! for the modern age!
 //!
 //! ## Usage
 //!
@@ -32,15 +33,15 @@
 //! # Cargo.toml
 //!
 //! [dependencies]
-//! windmark = "0.1.8"
+//! windmark = "0.1.9"
 //! tokio = { version = "0.2.4", features = ["full"] }
 //!
 //! # If you would like to use the built-in logger (recommended)
-//! # windmark = { version = "0.1.8", features = ["logger"] }
+//! # windmark = { version = "0.1.9", features = ["logger"] }
 //!
 //! # If you would like to use the built-in MIME dedection when `Success`-ing a file
 //! # (recommended)
-//! # windmark = { version = "0.1.8", features = ["auto-deduce-mime"] }
+//! # windmark = { version = "0.1.9", features = ["auto-deduce-mime"] }
 //! ```
 //!
 //! ### Implement a Windmark server
@@ -139,8 +140,8 @@ pub struct Router {
   error_handler:         Arc<Mutex<ErrorResponse>>,
   private_key_file_name: String,
   ca_file_name:          String,
-  header:                Arc<Mutex<Partial>>,
-  footer:                Arc<Mutex<Partial>>,
+  headers:               Arc<Mutex<Vec<Partial>>>,
+  footers:               Arc<Mutex<Vec<Partial>>>,
   ssl_acceptor:          Arc<SslAcceptor>,
   #[cfg(feature = "logger")]
   default_logger:        bool,
@@ -240,32 +241,40 @@ impl Router {
     self
   }
 
-  /// Set a header for the `Router` which should be displayed on every route.
+  /// Add a header for the `Router` which should be displayed on every route.
+  ///
+  /// # Panics
+  ///
+  /// May panic if the header cannot be added.
   ///
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_header(Box::new(|context| {
+  /// windmark::Router::new().add_header(Box::new(|context| {
   ///   format!("This is displayed at the top of {}!", context.url.path())
   /// }));
   /// ```
-  pub fn set_header(&mut self, handler: Partial) -> &mut Self {
-    self.header = Arc::new(Mutex::new(handler));
+  pub fn add_header(&mut self, handler: Partial) -> &mut Self {
+    (*self.headers.lock().unwrap()).push(handler);
 
     self
   }
 
-  /// Set a footer for the `Router` which should be displayed on every route.
+  /// Add a footer for the `Router` which should be displayed on every route.
+  ///
+  /// # Panics
+  ///
+  /// May panic if the header cannot be added.
   ///
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_footer(Box::new(|context| {
+  /// windmark::Router::new().add_footer(Box::new(|context| {
   ///   format!("This is displayed at the bottom of {}!", context.url.path())
   /// }));
   /// ```
-  pub fn set_footer(&mut self, handler: Partial) -> &mut Self {
-    self.footer = Arc::new(Mutex::new(handler));
+  pub fn add_footer(&mut self, handler: Partial) -> &mut Self {
+    (*self.footers.lock().unwrap()).push(handler);
 
     self
   }
@@ -336,7 +345,6 @@ impl Router {
     let mut response_mime_type = "".to_string();
     let mut footer = String::new();
     let mut header = String::new();
-    let content;
 
     while let Ok(size) = stream.read(&mut buffer).await {
       let content = String::from_utf8(buffer[0..size].to_vec())?;
@@ -372,43 +380,48 @@ impl Router {
       },
     ));
 
-    if let Ok(ref route) = route {
-      header = {
-        let header = (*self.header).lock().unwrap().call_mut((
-          RouteContext::new(stream.get_ref(), &url, &route.params),
-        ));
+    let content = if let Ok(ref route) = route {
+      let footers_length = (*self.footers.lock().unwrap()).len();
 
-        if header.is_empty() {
-          "".to_string()
-        } else {
-          format!("{}\n", header)
-        }
-      };
-      footer = {
-        let footer = (*self.footer).lock().unwrap().call_mut((
-          RouteContext::new(stream.get_ref(), &url, &route.params),
-        ));
-
-        if footer.is_empty() {
-          "".to_string()
-        } else {
-          format!("\n{}", footer)
-        }
-      };
-      content = {
-        to_value_set_status(
-          (*route.value).lock().unwrap().call_mut((RouteContext::new(
+      for partial_header in &mut *self.headers.lock().unwrap() {
+        header.push_str(&format!(
+          "{}\n",
+          partial_header(RouteContext::new(
             stream.get_ref(),
             &url,
             &route.params,
-          ),)),
-          &mut response_status,
-          #[cfg(not(feature = "auto-deduce-mime"))]
-          &mut response_mime_type,
-        )
-      };
+          )),
+        ));
+      }
+      for (i, partial_footer) in
+        (&mut *self.footers.lock().unwrap()).iter_mut().enumerate()
+      {
+        footer.push_str(&format!(
+          "{}{}",
+          partial_footer(RouteContext::new(
+            stream.get_ref(),
+            &url,
+            &route.params,
+          )),
+          if footers_length > 1 && i != footers_length - 1 {
+            "\n"
+          } else {
+            ""
+          },
+        ));
+      }
+      to_value_set_status(
+        (*route.value).lock().unwrap().call_mut((RouteContext::new(
+          stream.get_ref(),
+          &url,
+          &route.params,
+        ),)),
+        &mut response_status,
+        #[cfg(not(feature = "auto-deduce-mime"))]
+        &mut response_mime_type,
+      )
     } else {
-      content = to_value_set_status(
+      to_value_set_status(
         (*self.error_handler)
           .lock()
           .unwrap()
@@ -416,8 +429,8 @@ impl Router {
         &mut response_status,
         #[cfg(not(feature = "auto-deduce-mime"))]
         &mut response_mime_type,
-      );
-    }
+      )
+    };
 
     stream
       .write_all(
@@ -441,7 +454,7 @@ impl Router {
             _ => (&*content).to_string(),
           },
           match response_status {
-            20 => format!("{}{}{}", header, content, footer),
+            20 => format!("{}{}\n{}", header, content, footer),
             21 => (&*content).to_string(),
             _ => "".to_string(),
           }
@@ -781,8 +794,8 @@ impl Default for Router {
       }))),
       private_key_file_name: "".to_string(),
       ca_file_name: "".to_string(),
-      header: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
-      footer: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
+      headers: Arc::new(Mutex::new(vec![])),
+      footers: Arc::new(Mutex::new(vec![])),
       ssl_acceptor: Arc::new(
         SslAcceptor::mozilla_intermediate(SslMethod::tls())
           .unwrap()
